package org.radrails.rails.internal.ui.actions;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.IRailsUIConstants;

public class ScriptConsolePulldownDelegate implements
		IWorkbenchWindowPulldownDelegate {

	private IWorkbenchWindow fWindow;
	
	public void dispose() {

	}

	public void init(IWorkbenchWindow window) {
		fWindow = window;
	}

	public void run(IAction action) {
		runConsole("development");
	}
	
	private void runConsole(String env) {
		try {
			IProject project = RailsPlugin.getInstance().getSelectedRailsOrRubyProject();
			if(project != null) {
				ILaunchConfigurationWorkingCopy wc = RailsRuntime
						.createInternalLaunch(IRailsLaunchConfigAttributes.SCRIPT_RUBY, "console");
				wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, "script/console " + env);
				wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, project
						.getLocation().toOSString());
				wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
						IRailsLaunchConfigAttributes.SCRIPT_RUBY);
	
				ILaunchConfiguration config = wc.doSave();
				config.launch(ILaunchManager.RUN_MODE, null);
			}
			else {
				MessageDialog.openError(fWindow.getShell(),
						"Error opening console",
						IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);
			}
		} catch (CoreException e) {
			RailsLog.logError("Error running generator", e);
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {

	}

	public Menu getMenu(Control parent) {
		Menu m = new Menu(parent);
		createTestMenu(m);
		createProductionMenu(m);
		return m;
	}
	
	private void createTestMenu(Menu parent) {
		MenuItem test = new MenuItem(parent, SWT.PUSH);
		test.setText("script/console (test)");
		test.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				runConsole("test");
			}
		});
	}
	
	private void createProductionMenu(Menu parent) {
		MenuItem production = new MenuItem(parent, SWT.PUSH);
		production.setText("script/console (production)");
		production.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				runConsole("production");
			}
		});
	}

}
