package org.radrails.rails.internal.ui.editors.rhtml;

import java.io.ByteArrayInputStream;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.TextEditorAction;
import org.radrails.rails.internal.ui.RailsUILog;

public class ExtractPartialAction extends TextEditorAction {

	protected ExtractPartialAction(ResourceBundle bundle, String prefix, ITextEditor editor) {
		super(bundle, prefix, editor);
	}

	public void run() {
		ISelection sel = getTextEditor().getSelectionProvider().getSelection();
		if(sel != null && sel instanceof TextSelection && !sel.isEmpty()) {
			TextSelection textSel = (TextSelection) sel;
			InputDialog dialog = new InputDialog(getTextEditor().getSite().getShell(), "Extract Partial", "Enter the partial name:", "", null);
			if(dialog.open() == Dialog.OK) {
				String fileName = dialog.getValue();
				if(fileName != null && fileName.length() > 0) {
					IFileEditorInput fei = (IFileEditorInput) getTextEditor().getEditorInput();
					IPath path = fei.getFile().getFullPath().removeLastSegments(1).append(fileName);
					IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(path);
					if(!file.exists()) {
						try {
							ByteArrayInputStream bas = new ByteArrayInputStream(textSel.getText().getBytes());
							// create the partial file
							file.create(bas, true, null);
							// delete the partial text from the editor
							getTextEditor().getDocumentProvider().getDocument(getTextEditor().getEditorInput()).replace(textSel.getOffset(), textSel.getLength(), "");
						} catch (CoreException e) {
							RailsUILog.logError(e.getMessage(), e);
						} catch (BadLocationException e) {
							RailsUILog.logError(e.getMessage(), e);;
						}
					}
				}
			}
		}
	}

}
