package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ArrayList;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.radrails.rails.internal.ui.editors.rhtml.model.RHTMLNode;
import org.radrails.rails.internal.ui.editors.rhtml.model.TextNode;

/**
 * A content provider for a Treeview of an RHMTL document
 * 
 * @author brad
 */
public class RHTMLContentProvider implements ITreeContentProvider {
	public Object[] getChildren(Object parentElement) {
		Object[] children = null;
		if (parentElement instanceof RHTMLNode) {
			RHTMLNode rhtmlParent = (RHTMLNode) parentElement;
			children = trimWhitespaceNodes((RHTMLNode[]) rhtmlParent.getChildren());
		}
		return children;
	}

	public Object getParent(Object element) {
		Object res = null;
		if (element instanceof RHTMLNode) {
			res = ((RHTMLNode) element).getParent();
		}
		return res;
	}

	public boolean hasChildren(Object element) {
		boolean res = false;
		if (element instanceof RHTMLNode) {
			res = ((RHTMLNode) element).hasChildren();
		}
		return res;
	}

	public Object[] getElements(Object inputElement) {
		Object[] elements = null;
		if (inputElement instanceof RHTMLNode[] && inputElement != null) {
			elements = trimWhitespaceNodes((RHTMLNode[]) inputElement);
		}

		return elements;
	}

	/*
	 * Gets rid of the whitespace nodes so they don't clog up the outline tree
	 */
	private RHTMLNode[] trimWhitespaceNodes(RHTMLNode[] nodes) {
		ArrayList trimmed = new ArrayList(nodes == null ? 0 : nodes.length);
		for (int i = 0; nodes != null && i < nodes.length; i++) {
			if (!(nodes[i] instanceof TextNode)
					|| ((TextNode) nodes[i]).getText().trim().length() > 0) {
				trimmed.add(nodes[i]);
			}
		}

		return (RHTMLNode[]) trimmed.toArray(new RHTMLNode[0]);
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}
}
