/***************************** **************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/epl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import org.eclipse.jface.text.BadLocationException;
import org.radrails.rails.internal.ui.editors.rhtml.RHTMLParser.InvalidRHTMLException;
import org.radrails.rails.internal.ui.editors.rhtml.model.RHTMLNode;

/**
 * Formats RHTML code
 * 
 * @author brad
 */
public class RHTMLFormatter {

	/**
	 * Formats the given string of rhtml
	 * 
	 * @param unformatted
	 *            the rhtml to format
	 * @param spacesPerTab
	 *            how many spaces per tab char
	 * @param useTabs
	 *            whether to use tabs or spaces
	 * @param initialIndent
	 *            the initial indent of the current block. Set to null or "" if
	 *            no initial indent
	 * @return some nicely formatted rhtml
	 * @throws BadLocationException
	 */
	public static String format(String unformatted, int spacesPerTab, boolean useTabs, String initialIndent)
			throws BadLocationException, InvalidRHTMLException {
		String currentIndent = "";
		if (initialIndent != null && initialIndent.length() > 0) {
			currentIndent = initialIndent;
		}
		String indentString = getIndentString(spacesPerTab, useTabs);

		RHTMLParser parser = new RHTMLParser();
		RHTMLNode[] roots = parser.parse(unformatted);
		RHTMLFormatterVisitor visitor = new RHTMLFormatterVisitor(indentString);
		for (int i = 0; i < roots.length; i++) {
			visitor.visit(roots[i], currentIndent);
		}

		// Trim the leading new line we added
		String result = visitor.getFormatted();
		if (result.length() > 0 && result.substring(0, 1).equals(RHTMLNode.NEW_LINE)) {
			result = result.substring(1);
		}
		if (unformatted.endsWith(RHTMLNode.NEW_LINE)) {
			result += RHTMLNode.NEW_LINE;
		}
		return result;
	}

	/**
	 * Gets the indent string.
	 * 
	 * @param spacesPerTab
	 *            how many spaces to use per tab
	 * @param useTabs
	 *            whether to use tabs at all
	 */
	private static String getIndentString(int spacesPerTab, boolean useTabs) {
		String indentString = "";
		if (useTabs) {
			indentString = "\t";
		}
		else {
			indentString = "";
			for (int i = 0; i < spacesPerTab; i++) {
				indentString += " ";
			}
		}

		return indentString;
	}

}
