package org.radrails.rails.internal.ui.editors.rhtml.model;

import java.util.regex.Pattern;

import org.eclipse.jface.resource.ImageDescriptor;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.rhtml.RHTMLPartitionScanner;

/**
 * An embedded ruby node.
 * @author brad
 *
 */
public class ERbNode extends RHTMLNode {
	private static final ImageDescriptor OUTLINE_IMAGE = RailsUIPlugin.getImageDescriptor("icons/rubyfile.gif");

	private boolean fNeedsClose;
	
	public ERbNode(String text, int start) {
		super(text, start, RHTMLPartitionScanner.RHTML_RUBY_EXPRESSION);
		fNeedsClose = true;
	}
	
	public void setNeedsClose(boolean nc) {
		fNeedsClose = nc;
	}
	
	public boolean getNeedsClose() {
		return fNeedsClose;
	}

	public ImageDescriptor getImage() {
		return OUTLINE_IMAGE;
	}

	public String getLabel() {
		String label = getText().trim();
		if(label.length() > MAX_LABEL_LENGTH) {
			label = label.substring(0, MAX_LABEL_LENGTH);
			label += "...";
		}
		return label;
	}

	public boolean isOpenNode() {
		// match do blocks, if, unless, for, else, elsif
		/**
		 *  /<%-?.+?\s+do(\s*\|\w+\|)?\s*-?%>/
		 *  
		 *  /<%-?\s*if\s+.+?-?%>/
		 *  /<%-?\s*unless\s+.+?-?%>/
		 *  /<%-?\s*for\s+.+?\s+in\s+.+?-?%>/
		 *  
		 */
		Pattern doNode = Pattern.compile("<%-?.+?\\s+do(\\s*\\|\\w+\\|)?\\s*-?%>", Pattern.DOTALL);
		Pattern ifNode = Pattern.compile("<%-?\\s*if\\s+.+?-?%>", Pattern.DOTALL);
		Pattern unlessNode = Pattern.compile("<%-?\\s*unless\\s+.+?-?%>", Pattern.DOTALL);
		Pattern forNode = Pattern.compile("<%-?\\s*for\\s+.+?\\s+in\\s+.+?-?%>", Pattern.DOTALL);
		
		
		String str = getText().toLowerCase();
		return doNode.matcher(str).matches()
			|| ifNode.matcher(str).matches()
			|| unlessNode.matcher(str).matches()
			|| forNode.matcher(str).matches();
	}
	
	public boolean isIntermediateNode() {
		/** 
		 *  /<%-?\s*else\s*-?%>/
		 *  /<%-?\s*elsif\s*-?%>/
		 */
		Pattern elseNode = Pattern.compile("<%-?\\s*else\\s*-?%>", Pattern.DOTALL);
		Pattern elsifNode = Pattern.compile("<%-?\\s*elsif\\s*-?%>", Pattern.DOTALL);
		
		String str = getText().toLowerCase();
		return elseNode.matcher(str).matches()
			|| elsifNode.matcher(str).matches();
	}
	
	public boolean isCloseNode() {
		/**
		 *  /<%-?\s*end\s*-?%>/
		 */
		Pattern endNode = Pattern.compile("<%-?\\s*end\\s*-?%>", Pattern.DOTALL);
		
		String str = getText().toLowerCase();
		return endNode.matcher(str).matches();
	}
	
	/**
	 * Returns true if this ERb node is an output-style tag, e.g. <%= %> as opposed to
	 * the eval style like <% %> or <%- %> and false otherwise.
	 * 
	 * @return
	 */
	public boolean isOutputNode() {
		Pattern outputNode = Pattern.compile("<%=.+?%>", Pattern.DOTALL);
		String str = getText().toLowerCase();
		return outputNode.matcher(str).matches();
	}
	
	public boolean isMatchingCloseNode(RHTMLNode node) {
		// TODO offer else and elsif for open do blocks
		if(node instanceof ERbNode) {
			ERbNode erbNode = (ERbNode) node;
			return isCloseNode() && (erbNode.isOpenNode() || erbNode.isIntermediateNode());
		}
		return false;
	}
	
	public boolean isMatchingIntermediateNode(RHTMLNode node) {
		if(node instanceof ERbNode) {
			ERbNode erbNode = (ERbNode) node;
			return isIntermediateNode() && erbNode.isOpenNode();
		}
		return false;
	}
	
	public String getCloseNode() {
		// TODO handle -%> style tags
		return "<% end %>";
	}

}
