package org.radrails.rails.internal.ui.editors.rhtml.model;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.TreeNode;

/**
 * The base RHTMLNode type
 * @author brad
 *
 */
public abstract class RHTMLNode extends TreeNode {
	public static final String NEW_LINE = System.getProperty("line.separator");
	protected static final int MAX_LABEL_LENGTH = 10;
	
	private String type;
	private int start;

	public RHTMLNode(Object data, int start, String type) {
		super(data);
		this.type = type;
		this.start = start;
	}

	public String getType() {
		return type;
	}

	public abstract String getLabel();

	public abstract ImageDescriptor getImage();

	public String getText() {
		return (String) getValue();
	}

	public String getCloseNode() {
		return null;
	}

	/**
	 * Increment the indent size
	 */
	protected static String incrementIndent(String currentIndent, String indentString) {
		currentIndent += indentString;

		return currentIndent;
	}

	/**
	 * Decrement the indent size if possible. Leaves it alone if indent is empty
	 * 
	 */
	protected static String decrementIndent(String currentIndent, String indentString) {
		int indentLength = indentString.length();
		int currentIndentLength = currentIndent.length();
		if (currentIndentLength >= indentLength) {
			currentIndent = currentIndent.substring(0, currentIndentLength - indentLength);
		}

		return currentIndent;
	}

	protected String indent(String s, String currentIndent, boolean trim) {
		String[] lines = s.split(NEW_LINE);
		StringBuffer indented = new StringBuffer();

		for (int i = 0; i < lines.length; i++) {
			indented.append(currentIndent);
			String line = lines[i];
			if (trim) {
				line = line.trim();
			}
			indented.append(line);

			if (i < lines.length - 1) {
				indented.append(NEW_LINE);
			}
		}

		return indented.toString();
	}

	public int getStart() {
		return start;
	}

	public int getLength() {
		return getText().length();
	}

	public boolean equals(Object object) {
		boolean res = false;
		if (object instanceof RHTMLNode) {
			RHTMLNode other = (RHTMLNode) object;
			res = other.getText().equals(getText()) && other.getStart() == getStart();
		}
		return res;
	}

	/**
	 * Returns the complete text of the node. For most node this will be the
	 * same as getText(), but it allows for tags such as &lt;div&gt;s to include
	 * their children as well.
	 * 
	 * @return the complete text of this tag and all it's children
	 */
	public String getCompleteText() {
		return getText();
	}
}
